@@ -19,7 +19,7 @@ formats are generic enough to be used by any 2-D game.  Their purpose is
 to unify level editing.
 """
 
-__version__ = "0.3.1"
+__version__ = "0.4"
 
 
 import json
@@ -50,22 +50,55 @@ class ASCL(object):
     represent an object except for ``" "``, which represents an empty
     tile.
 
+    The width and height of the level is inferred from the number of
+    columns and rows in the level object grid, respectively.
+
+    .. attribute:: width
+
+       The width of the level in tiles.
+
+    .. attribute:: height
+
+       The height of the level in tiles.
+
     .. attribute:: meta
 
        A list of the level's meta variables.
 
+       .. note::
+
+          The meta variables can be any value, but when the ASCL is
+          saved, all meta variables will be automatically converted to
+          strings.
+
     .. attribute:: objects
 
        A list of objects in the level as :class:`jsl.LevelObject`
        objects.  They must have single-character strings as types.
+       Their positions are tile-based, and any meta-variables are
+       ignored when saving.
+
+       .. note::
+       
+          Due to the nature of this format, only integer positions are
+          valid.  If an object's position attribute is not an integer,
+          it will be rounded when the file is saved.
+
+       .. note::
+
+          Due to the nature of this format, only one object can be in
+          any given tile position.  If two objects are in the same tile
+          position, one will be arbitrarily lost when the file is saved.
     """
 
     def __init__(self):
+        self.width = 0
+        self.height = 0
         self.meta = []
         self.objects = []
 
     @classmethod
-    def load(cls, fname):
+    def load(cls, f):
         """
         Load the indicated file and return a :class:`ulvl.ASCL` object.
         """
@@ -77,6 +110,8 @@ class ASCL(object):
         y = 0
         for line in data.splitlines():
             if grid:
+                self.width = max(self.width, len(line))
+                self.height = max(self.height, y + 1)
                 for x in range(len(line)):
                     char = line[x]
                     if char != " ":
@@ -92,7 +127,7 @@ class ASCL(object):
 
     def save(self, f):
         """Save the object to the indicated file."""
-        objects = []
+        objects = [[' '] * self.width for i in range(self.height)]
 
         for obj in self.objects:
             T = str(obj.type)
@@ -138,6 +173,21 @@ class JSL(object):
       - The vertical position of the object in the level.
       - (Optional) An extra option for the object.  Can be any value.
 
+    - ``"size"``: An array with the following values:
+
+      - The width of the level.
+      - The height of the level.
+
+    .. attribute:: width
+
+       The width of the level.  The unit of measurement used is
+       arbitrary; pixels or tiles is recommended.
+
+    .. attribute:: height
+
+       The height of the level.  The unit of measurement used is
+       arbitrary; pixels or tiles is recommended.
+
     .. attribute:: meta
 
        A dictionary of all of the level's meta variables.
@@ -149,6 +199,8 @@ class JSL(object):
     """
 
     def __init__(self):
+        self.width = 0
+        self.height = 0
         self.meta = {}
         self.objects = []
 
@@ -161,6 +213,8 @@ class JSL(object):
 
         data = json.load(f)
 
+        self.width, self.height = data.get("size", [0, 0])
+
         self.meta = data.get("meta", {})
 
         for t in data.setdefault("objects", {}):
@@ -172,7 +226,8 @@ class JSL(object):
 
     def save(self, f):
         """Save the object to the indicated file."""
-        data = {"meta": self.meta, "objects": {}}
+        data = {"size": [self.width, self.height], "meta": self.meta,
+                "objects": {}}
 
         for obj in self.objects:
             data["objects"].setdefault(obj.type, [])
@@ -196,6 +251,8 @@ class ULX(object):
     A ULX file contains a root tree with the name "level" containing the
     following children:
 
+    - ``width``: the width of the level.
+    - ``height``: the height of the level.
     - ``meta``: contains one element for each of the level's meta
       variables.  Each element's tag indicates the name of the meta
       variable, while its text indicates the value.
@@ -207,6 +264,16 @@ class ULX(object):
       - ``"y"``: the vertical position of the object in the level.
       - ``"option"`` (optional): an extra option for the object.
 
+    .. attribute:: width
+
+       The width of the level.  The unit of measurement used is
+       arbitrary; pixels or tiles is recommended.
+
+    .. attribute:: height
+
+       The height of the level.  The unit of measurement used is
+       arbitrary; pixels or tiles is recommended.
+
     .. attribute:: meta
 
        A dictionary of all of the level's meta variables.
@@ -231,6 +298,8 @@ class ULX(object):
     """
 
     def __init__(self):
+        self.width = 0
+        self.height = 0
         self.meta = {}
         self.objects = []
 
@@ -245,7 +314,17 @@ class ULX(object):
         root = tree.getroot()
 
         for child in root:
-            if child.tag == "meta":
+            if child.tag == "width":
+                w = child.text
+                self.width = float(w)
+                if self.width == int(self.width):
+                    self.width = int(self.width)
+            elif child.tag == "height":
+                h = child.text
+                self.height = float(h)
+                if self.height == int(self.height):
+                    self.height = int(self.height)
+            elif child.tag == "meta":
                 for meta in child:
                     self.meta[meta.tag] = meta.text
             elif child.tag == "objects":
@@ -266,10 +345,18 @@ class ULX(object):
         """Save the object to the indicated file."""
         root = ET.Element("level")
 
+        width_elem = ET.Element("width")
+        width_elem.text = str(self.width)
+        root.append(width_elem)
+
+        height_elem = ET.Element("height")
+        height_elem.text = str(self.height)
+        root.append(height_elem)
+
         meta_elem = ET.Element("meta")
         for i in self.meta:
             elem = ET.Element(i)
-            elem.text = self.meta[i]
+            elem.text = str(self.meta[i])
             meta_elem.append(elem)
 
         root.append(meta_elem)
@@ -291,7 +378,7 @@ class ULX(object):
         root.append(objects_elem)
 
         tree = ET.ElementTree(root)
-        tree.write(f, xml_declaration=True)
+        tree.write(f, encoding="UTF-8", xml_declaration=True)
 
 
 class LevelObject(object):
